package crossmatch;

import javax.servlet.*;
import javax.servlet.http.*;
import org.apache.log4j.Logger;
import java.io.IOException;
import java.net.MalformedURLException;
import java.io.PrintWriter;
import java.io.InputStream;

/** Accept an HTTP GET request from the client.  Determine the request type (SCS query or Sesame lookup) and instatiate appropriate classes to fulfull the request.  Send the SCS or Sesame response to the client.
  */
public class Crossmatch extends HttpServlet {

    public static final String version = "0.2.4";
        
    private static Logger logger;
    
    public Parameters params;
    
    /** Servlet interface to crossmatch.  Accept an HTTP Get request. */
    public void doGet(HttpServletRequest request, HttpServletResponse response)  {
        logger = Logger.getLogger(Crossmatch.class);
        startLogMsg(logger);
        logger.debug("Received HTTP GET request");
        
        // Check for Sesame request
        if (request.getParameterValues("SESAME") != null) {
            logger.debug("GET request is for sesame lookup.");
            String identifier = request.getParameterValues("SESAME")[0];
            try {
                // Handle a Sesame lookup request
                Sesame sesame = new Sesame(identifier);
                // Create response for client
                response.setContentType(sesame.getContentType());
                PrintWriter pw = response.getWriter();
                pw.write(sesame.getContent());
                pw.close();
            }
            catch (MalformedURLException e) {
                logger.error("Inappropriate URL", e);
            }
            catch (IOException e) {
                logger.error("Error getting writer to response object.", e);
            }
        }

        // Request a VOTable by explicit URL
        else if ( ( request.getParameterValues("url") != null ) && 
                  ( ! request.getParameterValues("url")[0].equals("") ) ) {
            String url = request.getParameterValues("url")[0];
            logger.debug("GET request is for url:\n  " + url);
            try {
                UrlRequest urlRequest = new UrlRequest(url);
                urlRequest.fillResponse(response);
            }
            catch (MalformedURLException e) {
                logger.error("Inappropriate URL", e);
            }
            catch (IOException e) {
                logger.error("Error getting writer to response object.", e);
            }
        }
        
        // Query SCS services
        else if (request.getParameterValues("scs") != null) { 
            try {
                params = new Parameters(request);
                SCSQuery scsQuery = new SCSQuery(params.service(), params.ra(), params.dec(), params.sr());
                response.setContentType("text/xml");
                ServletOutputStream outStream = response.getOutputStream();
                fillOutputStream(scsQuery.responseStream(), outStream);
                outStream.close();
            }
            catch (Exception ex) {
                // Put the exception message in the http response
                response.setContentType("text/plain");
                logger.fatal("Exception thrown", ex);
                try {
                    PrintWriter pwTrace = response.getWriter();
                    ex.printStackTrace(pwTrace);
                    pwTrace.close();
                } catch (IOException ex2) {
                    logger.fatal("Exception thrown while tring to report previous exception through HTTP!", ex2);
                }
            }
        }
        stopLogMsg(logger);
    }

    /** Fill a servletOutputStream with the contents of an InputStream. */
    public static void fillOutputStream(InputStream inStream, ServletOutputStream outStream) throws IOException {
        int c;
        while ( (c = inStream.read()) != -1) {
            outStream.write( c );
        }
    }

    /** Write initial LoadTest logger message. **/
    private static void startLogMsg(Logger log) {
        log.debug("\n=================================\n" +
                    " Begin package crossmatch v" + version + "\n" +
                    "=================================");
    }
    /** Write final LoadTest logger message. **/
    private static void stopLogMsg(Logger log) {
        log.debug("\n===============================\n" +
                    " End package crossmatch v" + version + "\n" +
                    "===============================");
    }
}
