package crossmatch;

import org.apache.log4j.Logger;
import jhc.InputFile;
import jhc.Coordinate;
import java.util.HashMap;
import javax.servlet.http.*;
import java.util.ArrayList;
import java.util.regex.*;
import java.text.DecimalFormat;
import java.io.BufferedReader;
import java.io.FileReader;
import java.io.File;
import java.io.IOException;
import java.io.FileNotFoundException;

/** Obtain, hold, and provide the input parameters for the 
  * crossmatch package.
  **/
class Parameters {

    private Logger logger;
        
    // Servlet input Parameters
    private String serviceUrl;
    private double ra, dec, sr;
            
    /** Constructor for servlet mode. */
    public Parameters(HttpServletRequest request) throws Exception {
        
        logger = Logger.getLogger(this.getClass());
        logger.debug("Beginning Parameters(HttpServletRequest) constructor");
        
        // Get scs
        serviceUrl = request.getParameterValues("scs")[0];
        // Get ra, dec, sr
        ra  = convertRA(request.getParameterValues("ra")[0]);
        dec = convertDec(request.getParameterValues("dec")[0]);
        sr  = Double.parseDouble(request.getParameterValues("sr")[0]);
        logger.info( listParameters() );
    }
    
    /** Convert RA input to units of degrees */
    private double convertRA(String raString) throws Exception {
        ArrayList<Double> arrayList = parseCoordinate(raString);
        // If only one decimal number, assume it has units of degrees
        if (arrayList.size() == 1) 
            return formatDegree((double)arrayList.get(0));
        // If 2 or 3 decimal numbers, assume Hours:Minutes:Seconds format
        double h = arrayList.get(0);
        double m = arrayList.get(1);
        double s = 0;
        if (arrayList.size() == 3) s = arrayList.get(2);
        double degree = Coordinate.hms2deg(h, m, s);
        return formatDegree(degree);
    }
    
    /** Convert Dec input to units of degrees */
    private double convertDec(String decString) throws Exception {
        ArrayList<Double> arrayList = parseCoordinate(decString);
        // If only one decimal number, assume it has units of degrees
        if (arrayList.size() == 1) 
            return formatDegree((double)arrayList.get(0));
        // If 2 or 3 decimal numbers, assume Degrees:Arcminutes:Arcseconds format
        double d = arrayList.get(0);
        double m = arrayList.get(1);
        double s = 0;
        if (arrayList.size() == 3) s = arrayList.get(2);
        double degree = Coordinate.dms2deg(d, m, s);
        return formatDegree(degree);
    }
    
    /** Parse a coordinate string into groups of decimal values separated by
      * single characters.  A "+" or "-" may optionally be prepended to the 
      * string.
      */
    private ArrayList<Double> parseCoordinate(String s) throws Exception {
        logger.debug("String to parse: " + s);
        String regex = "[+-]?\\d*\\.?\\d+";
        Pattern pattern = Pattern.compile(regex);
        Matcher matcher = pattern.matcher(s);
        ArrayList<Double> arrayList = new ArrayList<Double>();
        while (matcher.find()) {
            arrayList.add(new Double( matcher.group() ));
            logger.debug("Double found: " + Double.toString(arrayList.get(arrayList.size()-1)));
        }
        if (arrayList.size() < 1 || arrayList.size() > 3) 
            throw new Exception("Error parsing coordinate value: " + s);
        return arrayList;
    }

    /** Format the degree value so it has a reasonable precision.  Too much 
      * precision causes problems for the SCS service.
      */
    private double formatDegree(double deg) {
        DecimalFormat f = new DecimalFormat("###.######");
        Double d = new Double(f.format(deg));
        logger.debug("Old degree format: " + (new Double(deg)).toString() );
        logger.debug("New degree format: " + d.toString());
        return d.doubleValue();
    }
        
    /** Return crossmatch version.
      **/
    public String getVersion() { return Crossmatch.version; }
    
    /** Return all paramters as a multi-line String. */
    public String listParameters() {
        String list = "Service URL = " + serviceUrl + "\n";
        list += "RA = " + ra + "\n";
        list += "Dec = " + dec + "\n";
        list += "Search Radius = " + sr;
        return list;
    }
    
    // Return private field values.
    public String service() { return serviceUrl; }
    public double ra() { return ra; }
    public double dec() { return dec; }
    public double sr() { return sr; }

    // Cone Search services:
    // VLAObsSummary
    public static final String vlaObsUrl =
        "https://webtest.aoc.nrao.edu/ivoa-dal/scsVlaObs?REQUEST=querydata&";
    // HST Archived Exposures Catalog
    public static final String hstaecUrl = 
        "http://heasarc.gsfc.nasa.gov/cgi-bin/vo/cone/coneGet.pl?table=hstaec&";
    // Chandra Archive Log
    public static final String chanALUrl = 
        "http://vizier.u-strasbg.fr/viz-bin/votable/-A?-source=B/chandra&";
    // NVSS
    public static final String nvssUrl = 
        "http://heasarc.gsfc.nasa.gov/cgi-bin/vo/cone/coneGet.pl?table=nvss&";

    // Default query params if none specified on command line.
    public static final String DEF_SERVICE1=nvssUrl, DEF_SERVICE2=nvssUrl;
    public static final double DEF_RA=12.0, DEF_DEC=12.0, DEF_SR=0.1;

}